/* BdfServer_JsonProducers - Copyright (c) 2012-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.ExportationDomain;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.balayage.engine.BalayageLog;
import fr.exemole.bdfserver.tools.configuration.PathConfigurationBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfCommandResultBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import java.util.LinkedHashSet;
import java.util.Set;
import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.exportation.balayage.BalayageContentDescription;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.TableExportContentDescription;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.tools.exportation.scrutari.ScrutariExportUtils;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.json.JsonUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.MessageByLineLog;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ExportationJsonProducerFactory {

    private ExportationJsonProducerFactory() {
    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        Lang lang = parameters.getWorkingLang();
        MessageLocalisation messageLocalisation = parameters.getMessageLocalisation();
        if (bdfCommandResult == null) {
            switch (json) {
                case ExportationDomain.SCRUTARIEXPORT_JSON:
                    MessageByLineLog messageByLineLog = ScrutariExportUtils.checkFieldGeneration(getScrutariExportDef(bdfServer, requestMap, null), bdfServer.getTableExportContext());
                    if (!messageByLineLog.getLogGroupList().isEmpty()) {
                        bdfCommandResult = BdfCommandResultBuilder.init()
                                .putResultObject(BdfInstructionConstants.MESSAGEBYLINELOG_OBJ, messageByLineLog)
                                .toBdfCommandResult();
                    }
            }
        }
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, parameters.getMessageLocalisation());
        switch (json) {
            case ExportationDomain.TABLEEXPORT_JSON:
                jsonProducer.add(new TableExportJsonProperty(getTableExportDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation));
                break;
            case ExportationDomain.TABLEEXPORT_ARRAY_JSON:
                bdfServer.getTableExportManager().update();
                jsonProducer.add(new TableExportArrayJsonProperty(bdfServer, lang, messageLocalisation));
                break;
            case ExportationDomain.TABLEEXPORT_CONTENT_JSON:
                jsonProducer.add(new TableExportContentJsonProperty(bdfServer, getTableExportContentDescription(bdfServer, requestMap), messageLocalisation));
                break;
            case ExportationDomain.SCRUTARIEXPORT_JSON:
                jsonProducer.add(new ScrutariExportJsonProperty(getScrutariExportDef(bdfServer, requestMap, bdfCommandResult), lang));
                break;
            case ExportationDomain.SCRUTARIEXPORT_ARRAY_JSON:
                jsonProducer.add(new ScrutariExportArrayJsonProperty(bdfServer, lang));
                break;
            case ExportationDomain.SCRUTARIEXPORT_PATHS_JSON:
                jsonProducer.add(new ScrutariExportPathsJsonProperty(getScrutariExportDef(bdfServer, requestMap, bdfCommandResult), getPathConfiguration(bdfServer, bdfCommandResult)));
                break;
            case ExportationDomain.TRANSFORMATION_DESCRIPTION_JSON:
                jsonProducer.add(new TransformationDescriptionJsonProperty(getTransformationDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation));
                break;
            case ExportationDomain.TRANSFORMATION_DESCRIPTIONS_JSON:
                bdfServer.getTransformationManager().update();
                jsonProducer.add(new TransformationDescriptionsJsonProperty(bdfServer.getTransformationManager(), getTransformationKeySet(bdfServer, requestMap), lang, messageLocalisation));
                break;
            case ExportationDomain.TRANSFORMATION_TEMPLATEDESCRIPTION_JSON:
                jsonProducer.add(new TemplateDescriptionJsonProperty(getTemplateDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation));
                break;
            case ExportationDomain.TRANSFORMATION_TEMPLATECONTENT_JSON:
                jsonProducer.add(new TemplateContentJsonProperty(bdfServer, getTemplateContentDescription(bdfServer, requestMap), messageLocalisation));
                break;
            case ExportationDomain.SQLEXPORT_JSON:
                jsonProducer.add(new SqlExportJsonProperty(getSqlExportDef(bdfServer, requestMap, bdfCommandResult), lang));
                break;
            case ExportationDomain.SQLEXPORT_ARRAY_JSON:
                jsonProducer.add(new SqlExportArrayJsonProperty(bdfServer, lang));
                break;
            case ExportationDomain.SQLEXPORT_PATHS_JSON:
                jsonProducer.add(new SqlExportPathsJsonProperty(getSqlExportDef(bdfServer, requestMap, bdfCommandResult), getPathConfiguration(bdfServer, bdfCommandResult)));
                break;
            case ExportationDomain.ACCESS_JSON:
                jsonProducer.add(new AccessJsonProperty(getAccessDef(bdfServer, requestMap, bdfCommandResult), lang));
                break;
            case ExportationDomain.ACCESS_ARRAY_JSON:
                jsonProducer.add(new AccessArrayJsonProperty(bdfServer, lang));
                break;
            case ExportationDomain.BALAYAGE_JSON:
                jsonProducer.add(new BalayageJsonProperty(getBalayageDescription(bdfServer, requestMap, bdfCommandResult), lang, messageLocalisation));
                break;
            case ExportationDomain.BALAYAGE_ARRAY_JSON:
                bdfServer.getBalayageManager().update();
                jsonProducer.add(new BalayageArrayJsonProperty(bdfServer, lang, messageLocalisation));
                break;
            case ExportationDomain.BALAYAGE_CONTENT_JSON:
                jsonProducer.add(new BalayageContentJsonProperty(bdfServer, getBalayageContentDescription(bdfServer, requestMap), messageLocalisation));
                break;
            case ExportationDomain.BALAYAGE_LOG_JSON:
                BalayageLog balayageLog = getBalayageLog(bdfCommandResult);
                if (balayageLog != null) {
                    jsonProducer.add(balayageLog);
                } else {
                    jsonProducer.add(JsonUtils.EMPTY_JSONPROPERTY);
                }
                break;
        }
        if (jsonProducer.isEmpty()) {
            return null;
        } else {
            return jsonProducer;
        }
    }

    private static BalayageDescription getBalayageDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        BalayageDescription balayageDescription = null;
        if (bdfCommandResult != null) {
            balayageDescription = (BalayageDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGEDESCRIPTION_OBJ);
        }
        if (balayageDescription == null) {
            balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(bdfServer.getBalayageManager(), requestMap);
        }
        return balayageDescription;
    }

    private static BalayageContentDescription getBalayageContentDescription(BdfServer bdfServer, RequestMap requestMap) throws ErrorMessageException {
        BalayageDescription balayageDescription = BdfInstructionUtils.getMandatoryBalayageDescription(bdfServer.getBalayageManager(), requestMap);
        return BdfInstructionUtils.getMandatoryBalayageContentDescription(balayageDescription, requestMap);
    }

    private static BalayageLog getBalayageLog(BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            return (BalayageLog) bdfCommandResult.getResultObject(BdfInstructionConstants.BALAYAGELOG_OBJ);
        }
        return null;
    }

    private static ScrutariExportDef getScrutariExportDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        ScrutariExportDef scrutariExportDef = null;
        if (bdfCommandResult != null) {
            scrutariExportDef = (ScrutariExportDef) bdfCommandResult.getResultObject(BdfInstructionConstants.SCRUTARIEXPORTDEF_OBJ);
        }
        if (scrutariExportDef == null) {
            scrutariExportDef = BdfInstructionUtils.getMandatoryScrutariExportDef(bdfServer.getScrutariExportManager(), requestMap);
        }
        return scrutariExportDef;
    }

    private static SqlExportDef getSqlExportDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        SqlExportDef sqlExportDef = null;
        if (bdfCommandResult != null) {
            sqlExportDef = (SqlExportDef) bdfCommandResult.getResultObject(BdfInstructionConstants.SQLEXPORTDEF_OBJ);
        }
        if (sqlExportDef == null) {
            sqlExportDef = BdfInstructionUtils.getMandatorySqlExportDef(bdfServer.getSqlExportManager(), requestMap);
        }
        return sqlExportDef;
    }

    private static AccessDef getAccessDef(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        AccessDef accessDef = null;
        if (bdfCommandResult != null) {
            accessDef = (AccessDef) bdfCommandResult.getResultObject(BdfInstructionConstants.ACCESSDEF_OBJ);
        }
        if (accessDef == null) {
            accessDef = BdfInstructionUtils.getMandatoryAccessDef(bdfServer.getAccessManager(), requestMap);
        }
        return accessDef;
    }

    private static TableExportDescription getTableExportDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        TableExportDescription tableExportDescription = null;
        if (bdfCommandResult != null) {
            tableExportDescription = (TableExportDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TABLEEXPORTDESCRIPTION_OBJ);
        }
        if (tableExportDescription == null) {
            tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
        }
        return tableExportDescription;
    }

    private static TableExportContentDescription getTableExportContentDescription(BdfServer bdfServer, RequestMap requestMap) throws ErrorMessageException {
        TableExportDescription tableExportDescription = BdfInstructionUtils.getMandatoryTableExportDescription(bdfServer.getTableExportManager(), requestMap);
        return BdfInstructionUtils.getMandatoryTableExportContentDescription(tableExportDescription, requestMap);
    }

    private static TransformationDescription getTransformationDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        TransformationDescription transformationDescription = null;
        if (bdfCommandResult != null) {
            transformationDescription = (TransformationDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TRANSFORMATIONDESCRIPTION_OBJ);
        }
        if (transformationDescription == null) {
            transformationDescription = BdfInstructionUtils.getMandatoryTransformationDescription(bdfServer, requestMap);
        }
        return transformationDescription;
    }

    private static PathConfiguration getPathConfiguration(BdfServer bdfServer, BdfCommandResult bdfCommandResult) {
        if (bdfCommandResult != null) {
            PathConfiguration pathConfiguration = (PathConfiguration) bdfCommandResult.getResultObject(BdfInstructionConstants.PATHCONFIGURATION_OBJ);
            if (pathConfiguration != null) {
                return pathConfiguration;
            }
        }
        return PathConfigurationBuilder.build(bdfServer);
    }

    private static Set<TransformationKey> getTransformationKeySet(BdfServer bdfServer, RequestMap requestMap) throws ErrorMessageException {
        String[] keysArray = BdfInstructionUtils.getMandatoryParameterValues(requestMap, "keys");
        Set<TransformationKey> transformationKeySet = new LinkedHashSet<TransformationKey>();
        for (String keys : keysArray) {
            String[] tokens = StringUtils.getTechnicalTokens(keys, true);
            for (String token : tokens) {
                try {
                    transformationKeySet.add(TransformationKey.parse(token));
                } catch (ParseException pe) {
                    throw BdfErrors.unknownParameterValue("keys", token);
                }
            }
        }
        return transformationKeySet;
    }

    private static TemplateDescription getTemplateDescription(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws ErrorMessageException {
        TemplateDescription templateDescription = null;
        if (bdfCommandResult != null) {
            templateDescription = (TemplateDescription) bdfCommandResult.getResultObject(BdfInstructionConstants.TEMPLATEDESCRIPTION_OBJ);
        }
        if (templateDescription == null) {
            templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(bdfServer, requestMap);
        }
        return templateDescription;
    }

    private static TemplateContentDescription getTemplateContentDescription(BdfServer bdfServer, RequestMap requestMap) throws ErrorMessageException {
        TemplateDescription templateDescription = BdfInstructionUtils.getMandatoryTemplateDescription(bdfServer, requestMap);
        return BdfInstructionUtils.getMandatoryTemplateContentDescription(templateDescription, requestMap);
    }

}
