/* FichothequeLib_Tools - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.fichotheque.importation.LiensImport;
import net.fichotheque.importation.ThesaurusImport;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.text.LabelChange;
import net.mapeadores.util.text.LabelChangeBuilder;
import net.mapeadores.util.text.LabelUtils;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusImportBuilder {

    private final InternalThesaurusImport thesaurusImport;
    private final Map<Integer, ThesaurusImport.MotcleImport> motcleMap = new HashMap<Integer, ThesaurusImport.MotcleImport>();
    private final Map<String, ThesaurusImport.MotcleImport> idalphaMotcleMap = new HashMap<String, ThesaurusImport.MotcleImport>();

    private ThesaurusImportBuilder(Thesaurus thesaurus, String type, Thesaurus destinationThesaurus) {
        this.thesaurusImport = new InternalThesaurusImport(thesaurus, type);
        thesaurusImport.destinationThesaurus = destinationThesaurus;
    }

    private ThesaurusImportBuilder(Thesaurus thesaurus, String type) {
        this.thesaurusImport = new InternalThesaurusImport(thesaurus, type);
    }

    public Thesaurus getThesaurus() {
        return thesaurusImport.getThesaurus();
    }

    public Thesaurus getDestinationThesaurus() {
        return thesaurusImport.getDestinationThesaurus();
    }

    public String getType() {
        return thesaurusImport.type;
    }

    public void addMerge(Motcle motcle, Motcle destinationMotcle) {
        if (!getType().equals(ThesaurusImport.MERGE_TYPE)) {
            throw new UnsupportedOperationException("type != ThesaurusImport.MERGE_TYPE");
        }
        int motcleid = motcle.getId();
        if (!motcleMap.containsKey(motcleid)) {
            ThesaurusImport.MotcleImport motcleImport = new InternalMergeMotcleImport(motcle, destinationMotcle);
            motcleMap.put(motcleid, motcleImport);
            thesaurusImport.add(motcleImport);
        }
    }

    public void addMove(Motcle motcle) {
        if (!getType().equals(ThesaurusImport.MOVE_TYPE)) {
            throw new UnsupportedOperationException("type != ThesaurusImport.MOVE_TYPE");
        }
        int motcleid = motcle.getId();
        if (!motcleMap.containsKey(motcleid)) {
            ThesaurusImport.MotcleImport motcleImport = new InternalMotcleImport(motcle);
            motcleMap.put(motcleid, motcleImport);
            thesaurusImport.add(motcleImport);
        }
    }

    public void addRemove(Motcle motcle) {
        if (!getType().equals(ThesaurusImport.REMOVE_TYPE)) {
            throw new UnsupportedOperationException("type != ThesaurusImport.REMOVE_TYPE");
        }
        int motcleid = motcle.getId();
        if (!motcleMap.containsKey(motcleid)) {
            ThesaurusImport.MotcleImport motcleImport = new InternalMotcleImport(motcle);
            motcleMap.put(motcleid, motcleImport);
            thesaurusImport.add(motcleImport);
        }
    }

    public void addChange(Motcle motcle, String newIdalpha, Object parent, LabelChangeBuilder changeLabelHolderBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder) {
        if (!getType().equals(ThesaurusImport.CHANGE_TYPE)) {
            throw new UnsupportedOperationException("type != ThesaurusImport.CHANGE_TYPE");
        }
        int motcleid = motcle.getId();
        InternalChangeMotcleImport motcleImport = (InternalChangeMotcleImport) motcleMap.get(motcleid);
        if (motcleImport == null) {
            motcleImport = new InternalChangeMotcleImport(motcle, newIdalpha, parent, changeLabelHolderBuilder, attributeChangeBuilder, liensImportBuilder);
            motcleMap.put(motcleid, motcleImport);
            thesaurusImport.add(motcleImport);
        } else {
            motcleImport.merge(changeLabelHolderBuilder, attributeChangeBuilder, liensImportBuilder);
        }
    }

    public void addCreation(int newId, String newIdalpha, int newParentId, String newParentIdalpha, LabelChangeBuilder changeLabelHolderBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder) {
        if (!getType().equals(ThesaurusImport.CREATION_TYPE)) {
            throw new UnsupportedOperationException("type != ThesaurusImport.CREATION_TYPE");
        }
        Thesaurus thesaurus = thesaurusImport.thesaurus;
        if (newId > 0) {
            InternalCreationMotcleImport current = (InternalCreationMotcleImport) motcleMap.get(newId);
            if (current != null) {
                current.merge(changeLabelHolderBuilder, attributeChangeBuilder, liensImportBuilder);
                return;
            }
            if (thesaurus.getMotcleById(newId) != null) {
                return;
            }
        } else {
            newId = -1;
        }
        if (thesaurus.isIdalphaType()) {
            if (newIdalpha == null) {
                return;
            }
            InternalCreationMotcleImport current = (InternalCreationMotcleImport) idalphaMotcleMap.get(newIdalpha);
            if (current != null) {
                current.merge(changeLabelHolderBuilder, attributeChangeBuilder, liensImportBuilder);
                return;
            }
        } else {
            newIdalpha = null;
            newParentIdalpha = null;
        }
        ThesaurusImport.MotcleImport motcleImport = new InternalCreationMotcleImport(newId, newIdalpha, newParentId, newParentIdalpha, changeLabelHolderBuilder, attributeChangeBuilder, liensImportBuilder);
        if (newId > 0) {
            motcleMap.put(newId, motcleImport);
        }
        if (newIdalpha != null) {
            idalphaMotcleMap.put(newIdalpha, motcleImport);
        }
        thesaurusImport.add(motcleImport);
    }

    public ThesaurusImport toThesaurusImport() {
        return thesaurusImport;
    }

    public static ThesaurusImportBuilder newInstance(Thesaurus thesaurus, String type, Thesaurus destinationThesaurus) {
        if (type.equals(ThesaurusImport.CHANGE_TYPE)) {
            return ThesaurusImportBuilder.newChangeInstance(thesaurus);
        }
        if (type.equals(ThesaurusImport.CREATION_TYPE)) {
            return ThesaurusImportBuilder.newCreationInstance(thesaurus);
        }
        if (type.equals(ThesaurusImport.MERGE_TYPE)) {
            return ThesaurusImportBuilder.newMergeInstance(thesaurus, destinationThesaurus);
        }
        if (type.equals(ThesaurusImport.MOVE_TYPE)) {
            return ThesaurusImportBuilder.newMoveInstance(thesaurus, destinationThesaurus);
        }
        if (type.equals(ThesaurusImport.REMOVE_TYPE)) {
            return ThesaurusImportBuilder.newRemoveInstance(thesaurus);
        }
        throw new IllegalArgumentException();
    }

    public static ThesaurusImportBuilder newMoveInstance(Thesaurus thesaurus, Thesaurus destinationThesaurus) {
        return new ThesaurusImportBuilder(thesaurus, ThesaurusImport.MOVE_TYPE, destinationThesaurus);
    }

    public static ThesaurusImportBuilder newMergeInstance(Thesaurus thesaurus, Thesaurus destinationThesaurus) {
        return new ThesaurusImportBuilder(thesaurus, ThesaurusImport.MERGE_TYPE, destinationThesaurus);
    }

    public static ThesaurusImportBuilder newCreationInstance(Thesaurus thesaurus) {
        return new ThesaurusImportBuilder(thesaurus, ThesaurusImport.CREATION_TYPE);
    }

    public static ThesaurusImportBuilder newChangeInstance(Thesaurus thesaurus) {
        return new ThesaurusImportBuilder(thesaurus, ThesaurusImport.CHANGE_TYPE);
    }

    public static ThesaurusImportBuilder newRemoveInstance(Thesaurus thesaurus) {
        return new ThesaurusImportBuilder(thesaurus, ThesaurusImport.REMOVE_TYPE);
    }


    private static class InternalThesaurusImport implements ThesaurusImport {

        private final Thesaurus thesaurus;
        private Thesaurus destinationThesaurus;
        private final String type;
        private final List<MotcleImport> motcleImportList = new ArrayList<MotcleImport>();

        private InternalThesaurusImport(Thesaurus thesaurus, String type) {
            this.thesaurus = thesaurus;
            this.type = type;
        }

        @Override
        public Thesaurus getThesaurus() {
            return thesaurus;
        }

        @Override
        public Thesaurus getDestinationThesaurus() {
            return destinationThesaurus;
        }

        @Override
        public String getType() {
            return type;
        }

        @Override
        public int getMotcleImportCount() {
            return motcleImportList.size();
        }

        @Override
        public MotcleImport getMotcleImport(int i) {
            return motcleImportList.get(i);
        }

        private void add(MotcleImport motcleImport) {
            motcleImportList.add(motcleImport);
        }

    }


    private static class InternalMotcleImport implements ThesaurusImport.MotcleImport {

        private final Motcle motcle;

        InternalMotcleImport(Motcle motcle) {
            this.motcle = motcle;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

    }


    private static class InternalMergeMotcleImport implements ThesaurusImport.MergeMotcleImport {

        private final Motcle motcle;
        private final Motcle destinationMotcle;

        private InternalMergeMotcleImport(Motcle motcle, Motcle destinationMotcle) {
            this.motcle = motcle;
            this.destinationMotcle = destinationMotcle;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

        @Override
        public Motcle getDestinationMotcle() {
            return destinationMotcle;
        }


    }


    private static class InternalChangeMotcleImport implements ThesaurusImport.ChangeMotcleImport {

        private final Motcle motcle;
        private final String newIdalpha;
        private final Object parent;
        private LabelChangeBuilder changeLabelHolderBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private LabelChange labelChange = null;
        private AttributeChange attributeChange = null;
        private LiensImport liensImport = null;

        private InternalChangeMotcleImport(Motcle motcle, String newIdalpha, Object parent, LabelChangeBuilder changeLabelHolderBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder) {
            this.motcle = motcle;
            this.newIdalpha = newIdalpha;
            this.parent = parent;
            this.changeLabelHolderBuilder = changeLabelHolderBuilder;
            this.attributeChangeBuilder = attributeChangeBuilder;
            this.liensImportBuilder = liensImportBuilder;
        }

        @Override
        public Motcle getMotcle() {
            return motcle;
        }

        @Override
        public String getNewIdalpha() {
            return newIdalpha;
        }

        @Override
        public Object getParent() {
            return parent;
        }

        @Override
        public LabelChange getLabelChange() {
            if (labelChange == null) {
                if (changeLabelHolderBuilder == null) {
                    labelChange = LabelUtils.EMPTY_LABELCHANGE;
                } else {
                    labelChange = changeLabelHolderBuilder.toLabelChange();
                }
            }
            return labelChange;
        }

        @Override
        public AttributeChange getAttributeChange() {
            if (attributeChange == null) {
                if (attributeChangeBuilder == null) {
                    attributeChange = AttributeUtils.EMPTY_ATTRIBUTECHANGE;
                } else {
                    attributeChange = attributeChangeBuilder.toAttributeChange();
                }
            }
            return attributeChange;
        }

        @Override
        public LiensImport getLiensImport() {
            if (liensImport == null) {
                if (liensImportBuilder == null) {
                    liensImport = ImportationUtils.EMPTY_LIENSIMPORT;
                } else {
                    liensImport = liensImportBuilder.toLiensImport();
                }
            }
            return liensImport;
        }

        private void merge(LabelChangeBuilder otherChangeLabelHolderBuilder, AttributeChangeBuilder otherAttributeChangeBuilder, LiensImportBuilder otherLiensImportBuilder) {
            if (otherChangeLabelHolderBuilder != null) {
                if (changeLabelHolderBuilder == null) {
                    changeLabelHolderBuilder = otherChangeLabelHolderBuilder;
                } else {
                    changeLabelHolderBuilder.merge(otherChangeLabelHolderBuilder);
                }
            }
            if (otherAttributeChangeBuilder != null) {
                if (attributeChangeBuilder == null) {
                    attributeChangeBuilder = otherAttributeChangeBuilder;
                } else {
                    attributeChangeBuilder.merge(otherAttributeChangeBuilder);
                }
            }
            if (otherLiensImportBuilder != null) {
                if (liensImportBuilder == null) {
                    liensImportBuilder = otherLiensImportBuilder;
                } else {
                    liensImportBuilder.merge(otherLiensImportBuilder);
                }
            }
        }

    }


    private static class InternalCreationMotcleImport implements ThesaurusImport.CreationMotcleImport {

        private final int newId;
        private final String newIdalpha;
        private final int newParentId;
        private final String newParentIdalpha;
        private LabelChangeBuilder changeLabelHolderBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private LabelChange labelChange = null;
        private AttributeChange attributeChange = null;
        private LiensImport liensImport = null;

        private InternalCreationMotcleImport(int newId, String newIdalpha, int newParent, String newParentIdalpha, LabelChangeBuilder changeLabelHolderBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder) {
            this.newId = newId;
            this.newIdalpha = newIdalpha;
            this.newParentId = newParent;
            this.newParentIdalpha = newParentIdalpha;
            this.changeLabelHolderBuilder = changeLabelHolderBuilder;
            this.attributeChangeBuilder = attributeChangeBuilder;
            this.liensImportBuilder = liensImportBuilder;
        }

        @Override
        public Motcle getMotcle() {
            return null;
        }

        @Override
        public int getNewId() {
            return newId;
        }

        @Override
        public String getNewIdalpha() {
            return newIdalpha;
        }

        @Override
        public int getParentId() {
            return newParentId;
        }

        @Override
        public String getParentIdalpha() {
            return newParentIdalpha;
        }

        @Override
        public LabelChange getLabelChange() {
            if (labelChange == null) {
                if (changeLabelHolderBuilder == null) {
                    labelChange = LabelUtils.EMPTY_LABELCHANGE;
                } else {
                    labelChange = changeLabelHolderBuilder.toLabelChange();
                }
            }
            return labelChange;
        }

        @Override
        public AttributeChange getAttributeChange() {
            if (attributeChange == null) {
                if (attributeChangeBuilder == null) {
                    attributeChange = AttributeUtils.EMPTY_ATTRIBUTECHANGE;
                } else {
                    attributeChange = attributeChangeBuilder.toAttributeChange();
                }
            }
            return attributeChange;
        }

        @Override
        public LiensImport getLiensImport() {
            if (liensImport == null) {
                if (liensImportBuilder == null) {
                    liensImport = ImportationUtils.EMPTY_LIENSIMPORT;
                } else {
                    liensImport = liensImportBuilder.toLiensImport();
                }
            }
            return liensImport;
        }

        private void merge(LabelChangeBuilder otherChangeLabelHolderBuilder, AttributeChangeBuilder otherAttributeChangeBuilder, LiensImportBuilder otherLiensImportBuilder) {
            if (otherChangeLabelHolderBuilder != null) {
                if (changeLabelHolderBuilder == null) {
                    changeLabelHolderBuilder = otherChangeLabelHolderBuilder;
                } else {
                    changeLabelHolderBuilder.merge(otherChangeLabelHolderBuilder);
                }
            }
            if (otherAttributeChangeBuilder != null) {
                if (attributeChangeBuilder == null) {
                    attributeChangeBuilder = otherAttributeChangeBuilder;
                } else {
                    attributeChangeBuilder.merge(otherAttributeChangeBuilder);
                }
            }
            if (otherLiensImportBuilder != null) {
                if (liensImportBuilder == null) {
                    liensImportBuilder = otherLiensImportBuilder;
                } else {
                    liensImportBuilder.merge(otherLiensImportBuilder);
                }
            }
        }

    }

}
